/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

package uk.co.petertribble.jangle;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.SocketException;
import snmp.*;
import java.io.IOException;

public class SnmpController {

    private SnmpParams params;
    private SNMPv1CommunicationInterface ifv1;
    private SnmpException savedException;

    public SnmpController(SnmpParams params) {
	this.params = params;
	initialize();
    }

    private void initialize() {
	try {
	    InetAddress hostAddress = InetAddress.getByName(params.getServer());
	    ifv1 =
		new SNMPv1CommunicationInterface(params.getVersion(),
						hostAddress,
						params.getCommunity());
	    // FIXME be specific
	} catch (UnknownHostException e) {
	    savedException = new SnmpException("Unknown host.");
	} catch (SocketException e) {
	    savedException = new SnmpException("Sockect error.");
	}
    }

    public SNMPv1CommunicationInterface getInterfaceV1() throws SnmpException {
	if (savedException != null) {
	    throw savedException;
	}
	return ifv1;
    }

    // FIXME throw the saved SnmpException if we're uninitialized
    public SNMPVarBindList getNextMIBEntry(String s) throws SnmpException {
	try {
	    return getInterfaceV1().getNextMIBEntry(s);
	} catch (IOException ioe) {
	    throw new SnmpException("IOException Retrieving " + s);
	} catch (SNMPBadValueException sbve) {
	    throw new SnmpException("SNMPBadValueException Retrieving " + s);
	} catch (SNMPGetException sge) {
	    throw new SnmpException("SNMPGetException Retrieving " + s);
	} catch (IndexOutOfBoundsException ioobe) {
	    // do nothing, we ran off the end
	}
	return null;
    }

    public SnmpObject getNext(String s) throws SnmpException {
	SNMPVarBindList newVars = getNextMIBEntry(s);
	if (newVars == null) {
	    return null;
	}
	SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
	SNMPObjectIdentifier snmpOID =
		(SNMPObjectIdentifier) pair.getSNMPObjectAt(0);
	SNMPObject snmpValue = pair.getSNMPObjectAt(1);
	return new SnmpObject(snmpOID, snmpValue);
    }

    public SNMPVarBindList getMIBEntry(String s) throws SnmpException {
	try {
	    return getInterfaceV1().getMIBEntry(s);
	} catch (IOException ioe) {
	    throw new SnmpException("IOException Retrieving " + s);
	} catch (SNMPBadValueException sbve) {
	    throw new SnmpException("SNMPBadValueException Retrieving " + s);
	} catch (SNMPGetException sbve) {
	    throw new SnmpException("SNMPGetException Retrieving " + s);
	}
    }

    public SnmpObject getValue(String s) throws SnmpException {
	SNMPVarBindList newVars = getMIBEntry(s);
	SNMPSequence pair = (SNMPSequence)(newVars.getSNMPObjectAt(0));
	SNMPObjectIdentifier snmpOID =
		    (SNMPObjectIdentifier) pair.getSNMPObjectAt(0);
	return new SnmpObject(snmpOID, pair.getSNMPObjectAt(1));
    }
}
